﻿using IdeaBlade.Aop;
using IdeaBlade.EntityModel;
using IdeaBlade.Validation;
using System;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Runtime.Serialization;

namespace WindowsPhoneDevTour {

  /*
   * Custom EntityManager and model classes.  See http://drc.ideablade.com/devforce-2012/bin/view/Documentation/code-first-entitymanager
   * 
   */


  /// <summary>
  /// Base class for all entity classes in model.  This class is decorated with the ProvideEntityAspect attribute,
  /// so CodeFirst infrastructure is also injected into sub-classes.
  /// See http://drc.ideablade.com/devforce-2012/bin/view/Documentation/code-first-entity-base-classes for more information.
  /// </summary>
  [ProvideEntityAspect]
  [DataContract(IsReference = true)]
  public abstract class BaseEntity {

    /// <summary>
    /// A getter will be injected during build.  See http://drc.ideablade.com/devforce-2012/bin/view/Documentation/code-first-entityaspect for more information.
    /// </summary>
    public EntityAspect EntityAspect { get { throw new NotImplementedException(); } }
  }

  /// <summary>
  /// The domain-specific EntityManager for your domain model.
  /// </summary>
  public partial class NorthwindIBEntities : EntityManager {

    /// <summary>Gets the <see cref="T:EntityQuery"/> for Customer entities. </summary>
    public EntityQuery<Customer> Customers { get; set; }
  }

  #region Customer class

  /// <summary>The auto-generated Customer class. </summary>
  [DataContract(IsReference = true)]
  public partial class Customer : BaseEntity {

    /// <summary>Gets or sets the CustomerID. </summary>
    [Key]
    [DataMember]
    [DatabaseGenerated(DatabaseGeneratedOption.None)]
    [RequiredValueVerifier(ErrorMessageResourceName = "Customer_CustomerID")]
    public Guid CustomerID { get; set; }

    /// <summary>Gets or sets the CustomerID_OLD. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 5, IsRequired = false, ErrorMessageResourceName = "Customer_CustomerID_OLD")]
    public string CustomerID_OLD { get; set; }

    /// <summary>Gets or sets the CompanyName. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 40, IsRequired = true, ErrorMessageResourceName = "Customer_CompanyName")]
    public string CompanyName { get; set; }

    /// <summary>Gets or sets the ContactName. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 30, IsRequired = false, ErrorMessageResourceName = "Customer_ContactName")]
    public string ContactName { get; set; }

    /// <summary>Gets or sets the ContactTitle. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 30, IsRequired = false, ErrorMessageResourceName = "Customer_ContactTitle")]
    public string ContactTitle { get; set; }

    /// <summary>Gets or sets the Address. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 60, IsRequired = false, ErrorMessageResourceName = "Customer_Address")]
    public string Address { get; set; }

    /// <summary>Gets or sets the City. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 15, IsRequired = false, ErrorMessageResourceName = "Customer_City")]
    public string City { get; set; }

    /// <summary>Gets or sets the Region. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 15, IsRequired = false, ErrorMessageResourceName = "Customer_Region")]
    public string Region { get; set; }

    /// <summary>Gets or sets the PostalCode. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 10, IsRequired = false, ErrorMessageResourceName = "Customer_PostalCode")]
    public string PostalCode { get; set; }

    /// <summary>Gets or sets the Country. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 15, IsRequired = false, ErrorMessageResourceName = "Customer_Country")]
    public string Country { get; set; }

    /// <summary>Gets or sets the Phone. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 24, IsRequired = false, ErrorMessageResourceName = "Customer_Phone")]
    public string Phone { get; set; }

    /// <summary>Gets or sets the Fax. </summary>
    [DataMember]
    [StringLengthVerifier(MaxValue = 24, IsRequired = false, ErrorMessageResourceName = "Customer_Fax")]
    public string Fax { get; set; }

    /// <summary>Gets or sets the RowVersion. </summary>
    [DataMember]
    public System.Nullable<int> RowVersion { get; set; }
  }
  #endregion Customer class
}
