﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using System.ComponentModel;
using IdeaBlade.EntityModel;
using WindowsPhoneDevTour.Resources;

namespace WindowsPhoneDevTour {
  public partial class DetailPage : PhoneApplicationPage, INotifyPropertyChanged {
    private Customer _customer;
    private bool _isBusy;

    public DetailPage() {
      InitializeComponent();

      DataContext = this;
    }

    public Customer Customer {
      get { return _customer; }
      set {
        if (Equals(value, _customer)) return;
        _customer = value;
        NotifyPropertyChange("Customer");
      }
    }

    public bool CanGoBack {
      get {
        return NavigationService != null && NavigationService.CanGoBack && !DataService.Instance.HasChanges();
      }
    }

    public bool CanSave {
      get { return DataService.Instance.HasChanges(); }
    }

    public bool CanDelete {
      get { return Customer != null && Customer.EntityAspect.EntityState != EntityState.Added; }
    }

    public bool CanDiscard {
      get { return DataService.Instance.HasChanges(); }
    }

    public bool IsBusy {
      get { return _isBusy; }
      private set {
        _isBusy = value;
        NotifyPropertyChange("IsBusy");
      }
    }

    #region INotifyPropertyChanged Members

    public event PropertyChangedEventHandler PropertyChanged;

    #endregion

    protected override void OnNavigatedTo(NavigationEventArgs e) {
      base.OnNavigatedTo(e);

      DataService.Instance.EntityChanged += OnEntityChanged;

      var id = NavigationContext.QueryString["id"];
      Start(new Guid(id));

      NotifyPropertyChange("CanGoBack");
    }

    protected override void OnNavigatedFrom(NavigationEventArgs e) {
      base.OnNavigatedFrom(e);

      DataService.Instance.EntityChanged -= OnEntityChanged;
    }

    private void OnEntityChanged(object sender, EntityChangedEventArgs entityChangedEventArgs) {
      NotifyPropertyChange("CanGoBack");
      NotifyPropertyChange("CanSave");
      NotifyPropertyChange("CanDiscard");
    }

    public async void Start(Guid customerId) {
      try {
        IsBusy = true;

        Customer = await DataService.Instance.GetCustomerAsync(customerId);
      } finally {
        NotifyPropertyChange("CanDelete");
        IsBusy = false;
      }
    }

    public void GoBack(object sender, EventArgs e) {
      NavigationService.GoBack();
    }

    private async void Save(object sender, EventArgs e) {
      try {
        IsBusy = true;

        await DataService.Instance.SaveAsync();
      } catch (EntityManagerSaveException err) {
        MessageBox.Show("Save failed: " + err.Message);
      } finally {
        NotifyPropertyChange("CanDelete");
        IsBusy = false;
      }
    }

    private async void Delete(object sender, EventArgs e) {
      var delete = MessageBox.Show("Are you sure you want to delete?", "", MessageBoxButton.OKCancel);
      if (delete == MessageBoxResult.OK) {
        try {
          IsBusy = true;
          await DataService.Instance.DeleteCustomerAsync(Customer);

          NavigationService.GoBack();
        } catch {
          DataService.Instance.Discard();

          MessageBox.Show("Customer cannot be deleted");
        } finally {
          IsBusy = false;
        }
      }
    }

    private void Discard(object sender, EventArgs e) {
      DataService.Instance.Discard();
      if (Customer.EntityAspect.EntityState == EntityState.Detached) {
        NavigationService.GoBack();
      }
    }

    private void NotifyPropertyChange(string propertyName) {
      var handler = PropertyChanged;
      if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
    }

    private void toggle_CanGoBack(object sender, DependencyPropertyChangedEventArgs e) {
      (ApplicationBar.Buttons[0] as ApplicationBarIconButton).IsEnabled = (sender as CheckBox).IsEnabled;
    }

    private void toggle_CanSave(object sender, DependencyPropertyChangedEventArgs e) {
      (ApplicationBar.Buttons[1] as ApplicationBarIconButton).IsEnabled = (sender as CheckBox).IsEnabled;
    }

    private void toggle_CanDelete(object sender, DependencyPropertyChangedEventArgs e) {
      (ApplicationBar.Buttons[2] as ApplicationBarIconButton).IsEnabled = (sender as CheckBox).IsEnabled;
    }

    private void toggle_CanDiscard(object sender, DependencyPropertyChangedEventArgs e) {
      (ApplicationBar.Buttons[3] as ApplicationBarIconButton).IsEnabled = (sender as CheckBox).IsEnabled;
    }
  }
}