﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using WindowsPhoneDevTour.Resources;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace WindowsPhoneDevTour {
  public partial class MainPage : PhoneApplicationPage, INotifyPropertyChanged {
    private ObservableCollection<Customer> _customers;
    private string _searchText;
    private Customer _selectedCustomer;
    private bool _isBusy;

    public MainPage() {
      InitializeComponent();

      DataContext = this;
    }

    public bool IsBusy {
      get { return _isBusy; }
      private set {
        _isBusy = value;
        NotifyPropertyChange("IsBusy");
      }
    }

    public ObservableCollection<Customer> Customers {
      get { return _customers; }
      set {
        if (Equals(value, _customers)) return;
        _customers = value;
        NotifyPropertyChange("Customers");
      }
    }

    public Customer SelectedCustomer {
      get { return _selectedCustomer; }
      set {
        _selectedCustomer = value;
        NotifyPropertyChange("SelectedCustomer");

        if (_selectedCustomer != null) {
          NavigateToDetailPage();
        }
      }
    }

    public string SearchText {
      get { return _searchText; }
      set {
        if (value == _searchText) return;
        _searchText = value;
        NotifyPropertyChange("SearchText");
      }
    }

    #region INotifyPropertyChanged Members

    public event PropertyChangedEventHandler PropertyChanged;

    #endregion

    protected override void OnNavigatedTo(NavigationEventArgs e) {
      base.OnNavigatedTo(e);

      Start();
    }

    public async void Start() {
      try {
        IsBusy = true;

        var customers = await DataService.Instance.GetAllCustomersAsync();
        Customers = new ObservableCollection<Customer>(customers);
      } catch (Exception ex) {
        var x = 1;
      } finally {
        IsBusy = false;
      }
    }

    private void NavigateToDetailPage() {
      NavigationService.Navigate(new Uri("/DetailPage.xaml?id=" + HttpUtility.UrlEncode(SelectedCustomer.CustomerID.ToString()), UriKind.Relative));
    }

    private void NewCustomer(object sender, EventArgs e) {
      SelectedCustomer = DataService.Instance.NewCustomer();
      NavigateToDetailPage();
    }

    private void NotifyPropertyChange(string propertyName) {
      var handler = PropertyChanged;
      if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
    }

    private async void Search(object sender, RoutedEventArgs e) {
      try {
        IsBusy = true;

        if (string.IsNullOrEmpty(SearchText)) {
          Start();
          return;
        }

        var customers = await DataService.Instance.FindCustomersAsync(SearchText);
        Customers = new ObservableCollection<Customer>(customers);
      } finally {
        IsBusy = false;
      }
    }
  }
}